<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license Creative Commons BY-NC-ND 4.0 <http://www.creativecommons.org/licenses/by-nc-nd/4.0/>
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Core;

/**
 * Basiccommands
 *
 * This Class includes all basic commands of this mod
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Commands extends Utils {

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "daemon";
    }

    /**
     * Constructs the class
     */
    public function __construct() {
        parent::__construct();
        $this->runPlugin();
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        //                          COMMAND       SYNTAX                    FUNCTION      OBJECT
        $this->mod->registerCommand("aliases", '~^aliases \S.*$~i', "aliases", $this);
        $this->mod->registerCommand("readconfig", false, "readConfig", $this);
        $this->mod->registerCommand("quitmod", false, "quit", $this);
        $this->mod->registerCommand("help", '~^help \S+$~i', "help", $this);
        $this->mod->registerCommand("cointoss", false, "cointoss", $this);
        $this->mod->registerCommand("time", false, "time", $this);
        $this->mod->registerCommand("kickall", false, "kickall", $this);
        $this->mod->registerCommand("uptime", false, "uptime", $this);
        $this->mod->registerCommand("info", false, "info", $this);
        $this->mod->registerCommand("me", false, "me", $this);
        $this->mod->registerCommand("rcon", '~^rcon .+$~i', "rcon", $this);
        $this->mod->registerCommand("say", '~^say .+$~i', "say", $this);
        $this->mod->registerCommand("pl", false, "pl", $this);
        $this->mod->registerCommand("cmdlist", false, "cmdlist", $this);
        $this->mod->registerCommand("protect", '~^protect \S.*$~i', "protect", $this);
        $this->mod->registerCommand("online", false, "online", $this);
        $this->mod->registerCommand("whisper", '~^whisper \S+ \S.*$~i', "whisper", $this);
        $this->mod->registerCommand("getip", '~^getip \S.*$~i', "getIp", $this);
    }

    /**
     * This Command return all aliases of a player
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return  void
     */
    public function aliases($guid, $parameters) {
        $search = implode(" ", $parameters);
        $player = $this->mod->findPlayerGuid($search);

        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($search)));
            return;
        }

        $aliases = $this->database->get('nicks', array('user' => $this->players[$player]->getUserId()), 'nick');

        $str = "";

        foreach ($aliases as $name) {
            $str .= "^3" . $name->nick . "^7, ";
        }

        $str = substr($str, 0, - 2);

        $this->players[$guid]->say($this->mod->getLngString("statsAliases") . $str);
    }

    /**
     * Reload the configs or a specific config
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function readConfig($guid, $parameters) {
        if (!empty($parameters)) {
            $config = "load/" . implode(" ", $parameters) . ".json";
            if (!$this->mod->readConfigFile($config)) {
                $this->players[$guid]->say($this->mod->getLngString("configFileLoadError", "<FILE>", $config));
                return false;
            }
            $this->players[$guid]->say($this->mod->getLngString("configFileLoad", "<FILE>", $config));
            return true;
        }
        $this->mod->readConfig();
        $this->rcon->rconSay($this->mod->getLngString("configParsed"));
    }

    /**
     * Quit the ManuAdminMod
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function quit($guid, $parameters) {
        $this->mod->killMod();
    }

    /**
     * Help returns a help string for player
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function help($guid, $parameters) {
        $this->mod->printHelp($parameters[0], $guid);
    }

    /**
     * cointoss return a message saying true or false
     *
     * THIS IS ONE OF THE BEST FEATURES OF THE ADMINMOD
     * OR DO YOU KNOW ANY OTHER ADMINTOOLS WHO HAVE SUCH A FEATURE???
     * SO, USE IT!!!!
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function cointoss($guid, $parameters) {
        $rand = rand(0, 1);
        $coin = array($this->mod->getLngString("heads"), $this->mod->getLngString("tails"));
        $this->rcon->rconSay($this->mod->getLngString("cointoss", array("<RESULT>"), array($coin[$rand])));
    }

    /**
     * returns a Message with the current time to the console
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function time($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("time", array("<TIME>"), array(date("H:i"))));
    }

    /**
     * kicks all players from the server
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function kickall($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("kickAll"));
        $this->rcon->rconKickAll();
    }

    /**
     * returns a message with the uptime to the server
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function uptime($guid, $parameters) {
        $uptime = \MAM\Daemon\Libraries\Helper\Helper::convertSeconds(time() - $this->registry->startuptime);
        $this->rcon->rconSay($this->mod->getLngString("uptime", array("<UPTIME>"), array($uptime)));
    }

    /**
     * info shows the currently used adminmod version on the server
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function info($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("versionInfo", array("<VERSION>"), array($this->registry->version)));
    }

    /**
     * returns the player a message with his PID and GUID
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function me($guid, $parameters) {
        $this->players[$guid]->say($this->mod->getLngString("me", array("<PID>", "<GUID>"), array($this->players[$guid]->getPID(), $guid)));
    }

    /**
     * The function to direclty execute a rcon command.
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function rcon($guid, $parameters) {
        if (!$this->permissions->hasPermissions($guid, 'daemon.rcon' . $parameters[0])) {
            $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
            return false;
        }
        $command = implode(" ", $parameters);
        $return = $this->rcon->rcon($command);
        $say = preg_replace('#(\r\n|\r|\n)#', "\n", $return);
        $say = implode("\n", array_slice(explode("\n", $say), 0, 6));
        $this->players[$guid]->say($say);
    }

    /**
     * says a message via console
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function say($guid, $parameters) {
        $parastring = implode(" ", $parameters);

        if (substr($parastring, 0, 1) === "@") {
            $parastring = substr($parastring, 1);
            $this->rcon->rconSay("^1" . $parastring);
        } else {
            $this->rcon->rconSay($this->players[$guid]->getName() . ": ^1" . $parastring);
        }
    }

    /**
     * shows a list with the pid and the name of the players on the server
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function pl($guid, $parameters) {
        $players = $this->mod->getPlayerList();
        $string = "";
        foreach ($players as $key => $value) {
            $string .= "^1$key^7=>^3$value^7; ";
        }
        $string = substr($string, 0, -1);
        $this->players[$guid]->say($string);
    }

    /**
     * shows the avaible commands of a player
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function cmdlist($guid, $parameters) {
        $commands = $this->mod->getGroupCommands($this->players[$guid]->getGroup());
        $say = $this->mod->getCV("daemon", "cmdprefix") . implode("; " . $this->mod->getCV("daemon", "cmdprefix"), $commands);
        $this->players[$guid]->say($say);
    }

    /**
     * shows the online admins which should be shown
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     */
    public function online($guid, $parameters) {
        $admins = $this->mod->getOnlineAdmins();
        $str = $this->mod->getLngString("onlineadmins") . " ";
        if (count($admins) > 0) {
            foreach ($admins as $admin_guid) {
                $str .= "^2" . $this->players[$admin_guid]->getName() . " ^1(" . $this->mod->getLongGroupName($this->players[$admin_guid]->getGroup()) . ")^7, ";
            }
            $str = substr($str, 0, -4);
        } else {
            $str .= $this->mod->getLngString("onlineAdminsNone");
        }
        $this->players[$guid]->say($str);
    }

    /**
     * tell a player a message
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function whisper($guid, $parameters) {
        $search = array_shift($parameters);
        $msg = implode(" ", $parameters);

        $player = $this->mod->findPlayerGuid($search);
        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($parameters)));
            return false;
        }

        $this->players[$player]->say($msg);
    }

    /**
     * get the ip of a player and saves it in a log
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return boolean
     */
    public function getIp($guid, $parameters) {
        $parameters = implode(" ", $parameters);

        $player = $this->mod->findPlayerGuid($parameters);
        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($parameters)));
            return false;
        }

        $ip = $this->players[$player]->getIP();

        if (!isset($ip)) {
            $this->players[$guid]->say($this->mod->getLngString("anyError"));
            return;
        }

        $this->players[$guid]->say($this->mod->getLngString("getIp", array("<NAME>", "<IP>"), array($this->players[$player]->getName(), $ip)));


        $timestamp = date("[d.m.y H:i:s]");
        $iplog = new \MAM\Daemon\Libraries\Logging\LogHandler("plugins/", "ips");
        $iplog->write($timestamp . " " . $this->players[$player]->getName() . ": " . $ip . " ($player)");
        unset($iplog);
    }
}
