<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Core;

/**
 * Player Class
 *
 * This class contains all player relevant functions/options
 *
 * @author mirko911
 * @Todo fix language string for warn
 */
class Player {
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Core\MOD
     */
    protected $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    protected $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     * @var \MAM\Daemon\Engine\Frostbite3\Player
     *
     */
    protected $players;
    /**
     * Databsae CLass
     *
     * @var \MAM\Daemon\Libraries\Database\Database
     */
    protected $database;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;
    /**
     * Guid of executing player
     *
     * @var  String $guid
     */
    protected $guid;
    /**
     * PID of the player
     * @var int
     */
    protected $pid;
    /**
     * IPv4 of a player
     *
     * @var string
     */
    protected $ip;
    /**
     * Name of a player
     *
     * @var string
     */
    protected $name;
    /**
     * Customvars
     *
     * is used by plugins to define custom variables for counters etc.
     *
     * @var array
     */
    protected $customvars = array();
    /**
     * Team of a player
     *
     * @var string
     */
    protected $team = 'none';
    /**
     * Unixtimestamp of playerjoin
     *
     * @var int
     */
    public $joined = 0;
    /**
     * Unixtimestamp of the last executed command
     *
     * @var int
     */
    public $lastCommand = 0;
    /**
     * Last death was teamkill
     *
     * @var boolean
     */
    public $lastdeathwastk = false;
    /**
     * Last kill was teamkill
     *
     * @var boolean
     */
    public $lastkillwastk = false;
    /**
     * Last player that was killed
     *
     * @var int
     */
    protected $lastkiller = false;
    /**
     * Last player that killed you
     *
     * @var int
     */
    protected $lastvictim = false;
    /**
     * Which weapon did you use for you last kill
     *
     * @var string
     */
    public $lastweaponkill = false;
    /**
     * Which weapon was used for you last death
     *
     * @var string
     */
    public $lastweapondeath = false;
    /**
     * Kill Counter
     *
     * @var int
     */
    public $kills = 0;
    /**
     * Death Counter
     *
     * @var int
     */
    public $deaths = 0;
    /**
     * Teamkill Counter
     * @var int
     */
    public $teamkills = 0;
    /**
     * Teamdeath counter
     *
     * @var int
     */
    public $teamdeaths = 0;
    /**
     * Selfkill Counter
     *
     * @var int
     */
    public $selfkills = 0;
    /**
     * Headshots Counter
     *
     * @var int
     */
    public $headshots = 0;
    /**
     * Punishment score
     * @var int
     */
    protected $punishmentScore = 0;
    /**
     * Info about the player from Database
     * @var object
     */
    protected $player_infos;
    /**
     * last punishment
     * @var string
     */
    protected $last_punishment;
    /**
     * Permissions Object
     *
     * with this object you can check if a player as enough permission to do s.th
     * @var \MAM\Daemon\Libraries\Permissions\Permissions
     */
    protected $permissions;

    /**
     * Constructor
     *
     * @param string $guid
     * @param int $pid
     * @param string $name
     */
    public function __construct($guid, $pid, $name) {
        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = & $registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
        $this->database = $registry->database;
        $this->permissions = $registry->permissions;

        $this->guid = strtolower($guid);
        $this->pid = $pid;
        $this->name = trim($name);
        $this->joined = time();

        $this->getDatabaseInformation();
    }

    /**
     * Cleanups the player object to get the destructor working
     *
     * It also sets the user offline in the database and recalculates the onlinetime
     */
    public function cleanup() {
        $onlinetime = time() - $this->joined;
        $sql = 'UPDATE <PFX>user SET onlinetime = onlinetime + ' . $onlinetime . ', online = 0 WHERE id = ' . $this->getUserId();
        $this->database->query($sql);

       foreach ($this as &$value) {
          unset($value);
       }
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return $this->name . ' (' . $this->guid . ')';
    }

    /**
     * Gets the stored informations about a player.
     *
     * This includes
     * <ul>
     * <li>Guid</li>
     * <li>Firstseen</li>
     * <li>Lastseen</li>
     * <li>firstip</li>
     * <li>lastip</li>
     * <li>connections</li>
     * <li>onlinetime</li>
     * </ul>
     *
     * It also updates the database with lastseen, lastip and connections
     */
    private function getDatabaseInformation() {
        $user = $this->database->row('user', array('guid' => $this->getGuid()));
        if (empty($user)) {
            $insert_array = array(
                'guid' => $this->getGuid(),
                'firstseen' => time(),
                'lastseen' => time(),
                'firstip' => $this->getIP(),
                'lastip' => $this->getIP(),
                'connections' => 1,
                'onlinetime' => 0,
                'online' => 1
            );
            $this->database->insert('user', $insert_array);
            $user = $this->database->row('user', array('guid' => $this->getGuid()));

            $allias_insert_array = array(
                'user' => $user->id,
                'nick' => $this->getName(),
            );
            $this->database->insert('nicks', $allias_insert_array);
        } else {

            $user = $this->database->row('user', array('guid' => $this->getGuid()));
            if (DEBUG) {
                $connections = $user->connections;
            } else {
                $connections = $user->connections + 1;
            }
            $insert_array = array(
                'lastseen' => time(),
                'lastip' => $this->getIP(),
                'connections' => $connections,
                'online' => 1
            );
            $this->database->update('user', $insert_array, array('id', '=', $user->id));

            $alias = $this->database->row('nicks', array('user' => $user->id, 'nick' => $this->getName()), '');
            if (empty($alias)) {
                $allias_insert_array = array(
                    'user' => $user->id,
                    'nick' => $this->getName(),
                );
                $this->database->insert('nicks', $allias_insert_array);
            }
        }
        $this->player_infos = $user;
    }

    /**
     * writes a customvar or  downwardly compatible team
     *
     * @param string $name the name of the var/team
     * @param string $value the value
     */
    public function __set($name, $value) {
        $this->customvars[$name] = $value;
    }

    /**
     * reads the custom var or the  downwardly compatible team
     *
     * @param string $name the name of the var/team
     * @return string returns the var
     */
    public function __get($name) {
        if (array_key_exists($name, $this->customvars)) {
            return $this->customvars[$name];
        } else {
            return false;
        }
    }

    /**
     * Checks if custom var exists
     *
     * @param string $name name of the var
     * @return boolean
     */
    public function __isset($name) {
        return array_key_exists($name, $this->customvars);
    }

    /**
     * unset a custom var
     *
     * @param string $name name of the custom var
     */
    public function __unset($name) {
        unset($this->customvars[$name]);
    }

    /**
     * Returns the last Victim of this Player [Victim = Opfer]
     * @return bool|string false or last Victim
     */
    public function getLastVictim() {
        if (!$this->lastvictim || !array_key_exists($this->lastvictim, $this->players)) {
            return false;
        }
        return $this->lastvictim;
    }

    /**
     * Returns the last Killer of this Palyer
     * @return boolean
     */
    public function getLastKiller() {
        if (!$this->lastkiller || !array_key_exists($this->lastkiller, $this->players)) {
            return false;
        }
        return $this->lastkiller;
    }

    /**
     * Says something to a Player. Use {{br}} for new Line
     * @param string $msg
     */
    public function say($msg) {
        $msg = str_replace('{{br}}', '\n', $msg);
        $msg = wordwrap($msg, 140 - strlen($this->mod->getCV('daemon', 'pmprefix')), '\n', true);
        $split = explode('\n', $msg);
        foreach ($split as $value) {
            $value = $this->mod->getCV('daemon', 'pmprefix') . $value;
            $this->rcon->rconTell($this->pid, $value);
        }
    }

    /**
     * Returns the group of a Player
     * @return string groupname
     */
    public function getGroup() {
        $admin = $this->mod->getAdminByGuid($this->guid);
        if (!$admin || !array_key_exists($admin['group'], $this->mod->getGroups())) {
            return 'default';
        }
        return $admin['group'];
    }

    /**
     * Returns the Playername
     * @return string Playername
     */
    public function getName() {
        return trim($this->name);
    }

    /**
     * Returns the PID of a Player
     * @return int PID
     */
    public function getPID() {
        return $this->pid;
    }

    /**
     * Returns the GUID of a Player
     * @return string Player Guid
     */
    public function getGuid() {
        return $this->guid;
    }

    /**
     * Returns the unique id from database
     * @return int
     */
    public function getUserId() {
        return $this->player_infos->id;
    }

    /**
     * Returns infomation from database.
     *
     * This information contains:
     *    int(11)    Nein
     *   guid    varchar(32)
     *   firstseen    int(11)
     *   lastseen    int(11)
     *   firstip    varchar(39)
     *   lastip    varchar(39)
     *   connections    int(11)
     *   onlinetime    int(11)
     *   online    tinyint(1)
     *
     * @return object
     */
    public function getPlayerInfos() {
        return $this->player_infos;
    }

    /**
     * check the commands, that the player can execute
     * @param string $command the command to check
     * @return bool 1 = can execute command 0 = not
     */
    public function isAllowedToExec($command) {
        return in_array($command, $this->mod->getGroupCommands($this->getGroup()));
    }

    /**
     * Sets this Player to a Group
     * @param string $group Groupname
     * @return boolean
     */
    public function setGroup($group) {
        if ($group == $this->getGroup()) {
            return true;
        }
        if ($group == 'default' && $this->getGroup() != 'default') {
            $this->mod->deleteAdmin($this->guid);
        } elseif ($group != 'default' && $this->getGroup() == 'default') {
            $this->mod->addAdmin($this->guid, $group, array($this->name), false);
        } else {
            $this->mod->updateAdmin($this->guid, array('group' => $group));
        }
    }

    /**
     * Returns the PbID of this Player. PBID = PID + 1
     * @return string PbID
     */
    public function getPbid() {
        return $this->pid + 1;
    }

    /**
     * returns punishment score
     * @return int
     */
    public function getPunishmentScore() {
        return $this->punishmentScore;
    }

    /**
     * Set punishment score
     * @param $score
     */
    public function setPunishmentScore($score) {
        $this->punishmentScore = $score;
    }

    /**
     * adds $add to punishment score
     * @param $add
     */
    public function punishmentScoreAdd($add) {
        $this->punishmentScore = $this->punishmentScore + $add;
    }

    /**
     * subtract $remove from punishment score
     * @param $remove
     */
    public function punishmentScoreRemove($remove) {
        $this->punishmentScore = $this->punishmentScore - $remove;
        if ($this->punishmentScore <= 0) {
            $this->punishmentScore = 0;
        }
    }

    /**
     * set the punishment score to zero
     */
    public function punishmentScoreClear() {
        $this->punishmentScore = 0;
    }

    /**
     * Warn a player
     * @param bool|string $reason
     * @param bool|string $kicker
     * @return bool
     */
    public function warn($reason = false, $kicker = false) {
        if ($this->permissions->hasPermissions($this->getGuid(), 'punishment.warn.ignore')) {
            if ($kicker != false) {
                $this->players[$kicker]->say($this->mod->getLngString('ignoreWarn'));
            }
            return false;
        }
        if (!empty($reason)) {
            $this->mod->findReason($reason);
        } else {
            $reason = $this->mod->getCV('punishment', 'defwarnreason');
        }
        $this->say($this->mod->getLngString('warnedByAdmin', array('<REASON>', '<SCORE>'), array($reason, $this->punishmentScore)));
        if ($this->mod->getCV('punishment', 'announce')) {
            $this->rcon->rconSay($this->mod->getLngString('playerWarnedPublicMsg', array('<REASON>', '<PLAYER>', '<ADMIN>'), array($reason, $this->getName(), ($kicker) ? $this->players[$kicker]->getName() : 'MOD')));
        }

        return true;
    }

    /**
     * Sets the last punishment
     * @param string $punishment
     */
    public function setLastPunishment($punishment) {
        $this->last_punishment = $punishment;
    }

    /**
     * Returns last punishment
     * @return string
     */
    public function getLastPunishment() {
        return $this->last_punishment;
    }
}
