<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Frostbite3\Bf4;

use MAM\Daemon\Core\Utils;

/**
 * Contains all game commands
 *
 * This class contains all game commands for battlefield 4
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Commands extends Utils {

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "daemon";
    }

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();
        $this->runPlugin();
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerCommand("maps", false, "maps", $this);
        $this->mod->registerCommand("map", '~^map [\w. -]+$~i', "map", $this);
        $this->mod->registerCommand("gametype", '~^gametype [\w. -]+$~i', "gametype", $this);
        $this->mod->registerCommand("roundrotate", false, "roundRotate", $this);
        $this->mod->registerCommand("yell", '~^yell .+$~i', "yell", $this);
        $this->mod->registerCommand("wyell", '~^wyell \S+ \S.*$~i', "wyell", $this);
        $this->mod->registerCommand("switchplayer", '~^switchplayer \S+ \S.*$~i', "switchplayer", $this);
        $this->mod->registerCommand("endround", '~^endround .+$~i', "endround", $this);
        $this->mod->registerCommand("kill", '~^kill \S.*$~i', "kill", $this);
        $this->mod->registerCommand("ff", false, "ff", $this);
        $this->mod->registerCommand("setff", '~^setff (0|1|2|3)$~i', "setff", $this);
        $this->mod->registerCommand("nextmap", false, "nextmap", $this);
        $this->mod->registerCommand("maprotate", false, "mapRotate", $this);
        $this->mod->registerCommand("team", false, "team", $this);
        $this->mod->registerCommand("unban", '~^unban \S.*$~i', "unban", $this);
        $this->mod->registerCommand("setnextmap", '~^setnextmap [\w. -]+$~i', "setnextmap", $this);
    }

    /**
     * shows the usable maps
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function maps($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getMaps("; "));
    }

    /**
     * Change the current map
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters part of mapname
     */
    public function map($guid, $parameters) {
        $parameters = array_map("strtolower", $parameters);
        $gamemode_check = $this->rcon->getMaps()->getShortGamemode($parameters[count($parameters) - 1]);
        if (count($parameters) > 1 && $gamemode_check !== false) {
            $gametype = $gamemode_check;
            unset($parameters[count($parameters) - 1]);
        } else {
            $gametype = $this->mod->getCurrentGametype();
        }

        $map = $this->mod->findMap(implode(" ", $parameters));

        if (!$map) {
            $this->players[$guid]->say($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
            return;
        }

        if (!$this->rcon->rconChangeMap($map, strtoupper($gametype))) {
            /**
             * @todo langString: gametypeNotAviableOnMap
             */
            $this->rcon->rconSay($this->mod->getLngString("gametypeNotAviableOnMap", array("<MAP>", "<GAMEMODE>"), array($map, $gametype)));
            return;
        }
        $this->rcon->rconSay($this->mod->getLngString("changeMap", array("<MAP>", "<GAMETYPE>"), array($this->mod->getLongMapName($map), $this->mod->getLongGametype($gametype))));
    }

    /**
     * Loads the next round in the rotation
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function roundRotate($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("loadingNextMap"));
        $this->rcon->rconRoundRotate();
    }

    /**
     * Loads the next map in the rotation
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function mapRotate($guid, $parameters) {
        $this->rcon->rconSay($this->mod->getLngString("loadingNextMap"));
        $this->rcon->rconMapRotate();
    }

    /**
     * returns a message with the next map to the console
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function nextmap($guid, $parameters) {
        $next = $this->rcon->rconGetNextMap();
        $map = $this->mod->getLongMapName($next["map"]);
        $gt = $this->mod->getLongGametype($next["gametype"]);
        $this->rcon->rconSay($this->mod->getLngString("nextmap", array("<GAMETYPE>", "<MAP>"), array($gt, $map)));
    }

    /**
     * gametype changes the gametype reload the current map with the selected gametype directly
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function gametype($guid, $parameters) {
        $gamemode = $this->rcon->getMaps()->getShortGamemode($parameters[0]);
        if ($gamemode === false) {
            $this->players[$guid]->say($this->mod->getLngString("gamemodeNotExist", array("<GAMETYPE>"), array($parameters[0])));
            return;
        }

        if ($this->rcon->rconMap($this->rcon->getMaps()->getMapName($this->rcon->getMaps()->getCurrentMapID()), $gamemode) == false) {
            $this->players[$guid]->say($this->mod->getLngString("notaviable", array("<MAP>", "<GAMETYPE>"), array($this->rcon->getMaps()->getMapName($this->rcon->getMaps()->getCurrentMapID()), $parameters[0])));
        } else {
            $this->rcon->rconSay($this->mod->getLngString("changeGametype", array("<GAMETYPE>"), array($this->mod->getLongGametype($parameters[0]))));
        }
        // $this->rcon->rconMapRestart(true);
    }

    /**
     * yell to all player
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function yell($guid, $parameters) {
        $this->rcon->rconYell(implode(" ", $parameters));
    }

    /**
     * Yell to one plyer
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function wyell($guid, $parameters) {
        $search = array_shift($parameters);
        $msg = implode(" ", $parameters);

        $player = $this->mod->findPlayerGuid($search);
        if (!$player) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($parameters)));
            return;
        }

        $this->players[$player]->yell($msg);
    }

    /**
     * set next map
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function setnextmap($guid, $parameters) {
        $parameters = array_map("strtolower", $parameters);
        $gamemode_check = $this->rcon->getMaps()->getShortGamemode($parameters[count($parameters) - 1]);
        if (count($parameters) > 1 && $gamemode_check !== false) {
            $gametype = $gamemode_check;
            unset($parameters[count($parameters) - 1]);
        } else {
            $gametype = $this->mod->getCurrentGametype();
        }

        $map = $this->mod->findMap(implode(" ", $parameters));

        if (!$map) {
            $this->players[$guid]->say($this->mod->getLngString("mapNotFound", array("<MAP>"), array(implode(" ", $parameters))));
            return;
        }

        $this->rcon->rconSetNextMap($map, $gametype, true);
        $this->rcon->rconSay($this->mod->getLngString("setNextMap", array("<MAP>", "<GAMETYPE>"), array($this->mod->getLongMapName($map), $this->mod->getLongGametype($gametype))));
    }

    /**
     * Executes an rcon command
     * @param String $guid Guid of executing player
     * @param String $parameters
     */
    public function rcon($guid, $parameters) {
        $command = implode(" ", $parameters);
        $return = $this->rcon->rcon($command);
        /* $this->rcon->rconSay($this->mod->getLngString("rconExecuted")); */
        if ($this->rcon->getStatus($return) === false) {
            $this->players[$guid]->say($return[0]);
            return;
        }
        foreach ($return[3] as $line) {
            $this->players[$guid]->say($line);
        }
    }

    /**
     * Switches a Player to a squad or team
     * @param String $guid Guid of executing player
     * @param String $parameters
     */
    public function switchPlayer($guid, $parameters) {
        $teams = $this->rcon->rconGetTeamnames();
        $to_switch = $this->mod->findPlayerGuid($parameters[1]);
        if (!$to_switch) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($parameters[1])));
            return;
        }
        $what = $parameters[0];
        switch ($what) {
            case "team":
                if (is_numeric($parameters[2])) {
                    $team_id = $parameters[2];
                    $this->rcon->rconSwitchPlayer($this->players[$to_switch], $team_id, 0, 1);
                } elseif (!is_numeric($parameters[2]) && in_array(strtoupper($parameters[2]), $teams)) {
                    $team_id = array_keys($teams, strtoupper($parameters[2]))[0];
                    $this->rcon->rconSwitchPlayer($this->players[$to_switch], $team_id, 0, (isset($parameters[3]) ? $parameters[3] : 1));
                } else {
                    $this->players[$guid]->say($this->mod->getLngString("teamInvalid", array("<TEAM>"), array($parameters[2])));
                    return;
                }

                break;
            case "squad":
                if (is_numeric($parameters[2])) {
                    $squad_id = $parameters[2];
                } elseif (!is_numeric($parameters[2]) && in_array(strtoupper($parameters[2]), $this->rcon->rconGetSquadnames())) {
                    $squad_id = array_keys($this->rcon->rconGetSquadnames(), strtoupper($parameters[2]))[0];
                } else {
                    $this->players[$guid]->say($this->mod->getLngString("squadInvalid", array("<SQUAD>"), array($parameters[2])));
                    return;
                }
                $this->rcon->rconSwitchPlayer($this->players[$to_switch], $this->players[$to_switch]->getTeam(), $squad_id, 1);
                break;
            default:
                $this->players[$guid]->say($this->mod->getLngString("switchInvalid", array("<WHAT>"), array($parameters[0])));
                break;
        }
    }

    /**
     * ends the current round if possible
     * @param String $guid Guid of executing player
     * @param String $parameters
     */
    public function endRound($guid, $parameters) {
        if (isset($parameters[0]) && is_numeric($parameters[0])) {
            $teamid = $parameters[0];
        } elseif (isset($parameters[0]) && !is_numeric($parameters[0]) && in_array($parameters[0], $this->rcon->rconGetSquadnames())) {
            $teamid = array_keys($this->rcon->rconGetSquadnames(), strtoupper($parameters[0]))[0];
        } else {
            $teamid = 0;
        }

        if (!$this->rcon->maps()->endRound($teamid)) {
            $this->players[$guid]->say($this->mod->getLngString("endRoundError", array("<TEAM>"), array($parameters[0])));
            return;
        }
        $this->players[$guid]->say($this->mod->getLngString("endRoundSuccess", array(), array()));
    }

    /**
     * Kills a player
     * @param String $guid Guid of executing player
     * @param String $parameters
     */
    public function kill($guid, $parameters) {
        $parastring = implode(" ", $parameters);
        if (strpos($parastring, "\\") !== false) {
            list($player, $parameters) = explode("\\", $parastring, 2);
            $player = trim($player);
            $parameters = array(trim($parameters));
        } else {
            $player = array_shift($parameters);
        }
        $tokill = $this->mod->findPlayerGuid($player);

        if (!$tokill) {
            $this->players[$guid]->say($this->mod->getLngString("playerNotFound", array("<SEARCH>"), array($player)));
        } else {

            //Check if there is enough power
            if ($this->players[$tokill]->getNeededPower() > $this->players[$guid]->getPower()) {
                $this->players[$guid]->say($this->mod->getLngString("morePowerNeeded"));
                return;
            }

            if (!empty($parameters)) {
                $reason = implode(" ", $parameters);
                $this->mod->findReason($reason);
            } else {
                $reason = "Killed by admin";
            }
            if ($this->rcon->rconKillPlayer($this->players[$tokill])) {
                $this->players[$tokill]->say($reason);
            } else {
                $this->players[$guid]->say($this->mod->getLngString("anyError"));
            }
        }
    }

    /**
     * retuns a message that shows the friendly fire
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function ff($guid, $parameters) {
        $ff = $this->rcon->rcon("vars.friendlyFire")[1];
        $ff = ($ff == "false") ? 0 : 1;
        $mode = array(
            $this->mod->getLngString("disabled"),
            $this->mod->getLngString("enabled"),
        );
        $this->rcon->rconSay($this->mod->getLngString("friendlyFire", array("<MODE>"), array($mode[$ff])));
    }

    /**
     * sets the friendly fire to x
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     */
    public function setff($guid, $parameters) {
        $mode = array(
            $this->mod->getLngString("disabled"),
            $this->mod->getLngString("enabled"),
        );
        $this->rcon->rconSay($this->mod->getLngString("setFriendlyFire", array("<MODE>"), array($mode[$parameters[0]])));
        $this->rcon->rcon("vars.friendlyFire $parameters[0]");
    }

    /**
     * Sends team status
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     * @todo Maybe just fun command
     */
    public function team($guid, $parameters) {
        $teams = $this->rcon->rconGetTeamnames();
        $squads = $this->rcon->getMaps()->getSquadnames();
        $this->players[$guid]->say("Your Team: " . $teams[$this->players[$guid]->getTeam()] . ", your Squad: " . $squads[$this->players[$guid]->getSquad()]);
    }

    /**
     * Unban a player
     * @param String $guid Guid of executing player
     * @param String $parameters Additional parameters
     * @todo Move to punishment?
     */
    public function unban($guid, $parameters) {
        $parastring = implode(" ", $parameters);
        $player = array_shift($parameters);
    }
}
