<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Quake3;

/**
 * Loop for frostbite3 engine
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Loop {

    /**
     * Initialise rcon for Cod
     */
    static function init() {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        /* @var $mod \MAM\Daemon\Core\Mod  */
        $mod = $registry->mod;
        $logging = $registry->logging;

        try {
            /* @var $rcon M\AM\Daemon\Engine\Quake3\Rcon\RconCommands  */
            $registry->rcon = $rcon = new Rcon\RconCommands();
            $mod->rcon = $rcon;

            //test rcon connection
            $return = $rcon->rcon("version");

            if (stripos($return, "Invalid password") !== false || stripos($return, "bad rcon") !== false) {
                throw new \Exception("RCON connection failed: Wrong RCON password");
            } elseif (stripos($return, "The server must set 'rcon_password'")) {
                throw new \Exception("RCON connection failed: There is no rcon password set on the server. Please set a rcon password in your server.cfg (set rcon_password PASSWORD)");
            } elseif (empty($return)) {
                throw new \Exception("RCON connection not valid, check whether your config (IP + port) is correct and your gameserver is running");
            }
        } catch (\Exception $e) {
            $logging->error($e->getMessage());
        }

        $logging->notice("RCON connection established (" . $mod->getCV("connection", "ip") . ":" . $mod->getCV("connection", "port") . ")");

        //Detect Game
        switch ($mod->getGame()) {
            case "cod5":
                $game = "Call of Duty 5";
                break;
            case "cod4":
                if ($registry->iceops) {
                    $game = "Call of Duty 4 (Iceops Files)";
                } else {
                    $game = "Call of Duty 4";
                }
                break;
            case "cod2":
                $game = "Call of Duty 2";
                break;
            default:
                $logging->error("Unknown game detected: ".$mod->getGame());
                break;
        }

        $logging->notice("Game detected: $game");


        $test = $rcon->rconDvarList("g_log");
        if ($test["g_logsync"] < 1) {
            $rcon->rconSetDvar("g_logSync", 1);
            $logging->warning("Dvar 'g_logsync' must be greater or equal to 1");
        } else {
            $logging->notice("Dvar check successful");
        }


        $punishment = new \MAM\Daemon\Libraries\Punishment\Punishment;

        $punishment->registerPunishment("warn", "warn", array("reason", "kicker"));
        $punishment->registerPunishment("ban", "ban", array("reason", "kicker"));
        $punishment->registerPunishment("kick", "kick", array("reason", "kicker"));
        $punishment->registerPunishment("tempban", "tempBan", array("reason", "kicker", "time"));

        $punishment->parseCfg();

        $registry->punishment = $punishment;
        new \MAM\Daemon\Libraries\Punishment\Commands();

        //Get Teamnames
        $mod->updateTeamNames();
        new \MAM\Daemon\Engine\Quake3\Commands();

        //Set Version info to gameservers status
        $mod->setVersionDvar();
    }

    /**
     *  Start final loop
     */
    static function loop() {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $mod = $registry->mod;
        $players = &$registry->players;
        $rcon = $registry->rcon;
        $logging = $registry->logging;

        //Now start parsing the log files
        //declare $parser object;

        $logfile = $mod->getCV("quake3", "logfile");
        if ($logfile === "auto") {
            $fsgame = $rcon->rconGetDvar("fs_game");
            $fsgame = ($fsgame !== "") ? $fsgame : "main";

            $logfile = $rcon->rconGetDvar('fs_homepath') . DIRECTORY_SEPARATOR . $fsgame . DIRECTORY_SEPARATOR . $rcon->rconGetDvar('g_log');
        }
        $parser = new \MAM\Daemon\Engine\Quake3\Parser($logfile, $success);
        $registry->parser = $parser;

        if (!$success) {
            if (!is_readable($logfile)) {
                $logging->error("Logfile is not readable: '" . $logfile . "'");
            }
            $logging->error("Couldn't open server's logfile: '" . $logfile . "'");
        }

        if (count($players) <= 6 || $mod->getCV("quake3", "antistartupbug") === false) {
            $waitOnNextMap = false;
        } else {
            $waitOnNextMap = true;
            $players = array();
        }
        $mapRestartPerformed = false;

        //Announcestartup
        $rcon->rconSay($mod->getLngString("watchingServer"));
        $mod->triggerEvent('mamStarted');

        while (true) {

            clearstatcache();

            $newlines = $parser->getNewLines();

            foreach ($newlines as $line) {
                $line = $parser->parseLine($line);
                if (!$line) {
                    continue;
                }

                if ($parser->getTimeStampType() == "absolute") {
                    $uptime = $parser->getLastParsedTimeStamp();
                } else {
                    $timestamp = $parser->getLastParsedTimeStamp();
                    $seconds = $timestamp % 60;
                    $minutes = ($timestamp - $seconds) / 60;
                    $uptime = "$minutes:" . str_pad($seconds, 2, "0", STR_PAD_LEFT);
                }

                if ($waitOnNextMap) {
                    if ($line["action"] != "nextmap") {
                        if ($mod->getCV("quake3", "antistartupbug") === true && !$mapRestartPerformed) {
                            $rcon->rconMapRestart(true);
                            $mapRestartPerformed = true;
                        }
                        continue;
                    } else {
                        $waitOnNextMap = false;
                    }
                }

                $mod->triggerEvent("logAction", $line);
                $action_handler = new ActionHandler();

                switch ($line["action"]) {
                    case "damage":
                        $action_handler->Damage($line["parsed"]);
                        break;

                    case "vehicledamage":
                        //Damage given to Tanks
                        $action_handler->VehicleDamage($line["parsed"]);
                        break;

                    case "actorsdamage":
                        //Damage given to Dogs!
                        $action_handler->ActorsDamage($line["parsed"]);
                        break;

                    case "kill":
                        $action_handler->Kill($line["parsed"]);
                        break;

                    case "join":
                        $action_handler->Join($line["parsed"]);
                        break;

                    case "jointeam":
                        $action_handler->JoinTeam($line["parsed"]);
                        break;

                    case "quit":
                        $action_handler->Quit($line["parsed"]);
                        break;

                    case "say":
                    case "sayteam":
                        $action_handler->Say($line["parsed"]);
                        break;

                    case "nextmap":
                        $action_handler->NextMap($line["parsed"]);
                        break;

                    case "tie":
                    case "win":
                    case "loss":
                        $action_handler->Result($line["action"], $line["parsed"]);
                        break;

                    case "bombplanted":
                    case "bombdefused":
                    case "flagcaptured":
                    case "flagtaken":
                    case "flagreturned":
                    case "hqcaptures":
                    case "hqdestroyed":
                        $action_handler->Action($line["action"], $line["parsed"]);
                        break;

                    case "action":
                        $action_handler->ActionCoD2($line["parsed"]);
                        break;

                    case "mapend":
                        break;

                    default: continue; //Empty lines e.g.
                }
            }

            $mod->triggerEvent("everyTime");
            usleep(10000);
        }
    }
}
