<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Libraries\Punishment;

use MAM\Daemon\Core\Registry;

/**
 * Punishment DB-Class
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class Database {
    /**
     * Database object
     *
     * @var \MAM\Daemon\Libraries\Database\Database
     */
    private $database;
    /**
     * Mod
     *
     * @var \MAM\Daemon\Core\Mod
     */
    private $mod;
    /**
     * Players
     *
     * @var \MAM\Daemon\Core\Player
     */
    private $players;
    /**
     * Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;

    /**
     * Constructs the class
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = & $registry->players;
        $this->logging = $registry->logging;
        $this->database = $registry->database;
    }

    /**
     * Destruct the class
     */
    public function __destruct() {
        array_map(array($this, "saveScore"), array_keys($this->players));
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "Punishment DB-Class";
    }

    /**
     * Create an user for the punishment score
     * @param String $guid
     */
    public function createUser($guid) {
        $user = $this->database->row('punishmentScores', array('user' => $this->players[$guid]->getUserId()));
        if (empty($user)) {
            if (!$this->database->insert('punishmentScores', array('user' => $this->players[$guid]->getUserId(), 'score' => 0, 'lastupdate' => time()))) {
                $this->logging->warning($this . ': Can \'t create user ' . $this->players[$guid]->getUserId());
            }
        }
    }

    /**
     * Returns the punishment score
     * @param String $guid
     * @return array
     */
    public function getScore($guid) {
        $result = $this->database->row('punishmentScores', array('user' => $this->players[$guid]->getUserId()), 'score');
        return $result->score;
    }

    /**
     * Returns the last update of the punishment score
     * @param String $guid
     * @return array
     */
    public function getLastUpdate($guid) {
        $result = $this->database->row('punishmentScores', array('user' => $this->players[$guid]->getUserId()), 'lastupdate');
        return $result->lastupdate;
    }

    /**
     * save the punishment score of a player
     *
     * @param String $guid
     */
    public function saveScore($guid) {
        $score = $this->players[$guid]->getPunishmentScore();

        $insert = array(
            "score" => $score,
            "lastupdate" => time()
        );
        $this->database->update('punishmentScores', $insert, array('user', '=', $this->players[$guid]->getUserId()));
    }

    /**
     * Saves score log
     * @param string $guid
     * @param string $reason
     * @param int $score_added
     */
    public function logScore($guid, $reason, $score_added) {
        $insert = array(
            "user" => $this->players[$guid]->getUserId(),
            "scoreAdded" => $score_added,
            "time" => time(),
            "isValid" => 1,
            "reason" => $reason
        );
        $this->database->insert('punishmentLog', $insert);
    }
}
