<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Plugins\Badwords;

use MAM\Daemon\Core\Utils;

/**
 * Init Players because playerJoined isn't called at startup
 * @todo Merge with Badnames
 */
class BadWords extends Utils {
    /**
     * Bad word list
     *
     * @var array
     */
    private $bad_word_list = array();

    /**
     * Constructor
     */
    public function __construct() {
        parent::__construct();

        $this->read();
    }

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV("badwords", "enabled", 0);
        $this->mod->setDefaultCV("badwords", "warnstokick", 3);
        $this->mod->setDefaultCV("badwords", "mode", "tempban");
        $this->mod->setDefaultCV("badwords", "kickreason", "Use of bad words");
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        array_map(array($this, "playerInit"), array_keys($this->players));

        $this->mod->registerEvent("playerSay", "main", $this);
        $this->mod->registerEvent("playerJoined", "playerInit", $this);
        $this->mod->registerEvent("parseConfig", "read", $this);
    }

    /**
     * Executes the playerSay event.
     *
     * @param array $args
     * @return void
     */
    public function main($args) {
        list($guid, $message, $executed) = $args;


        if (!$this->mod->getCV("badwords", "enabled")) {
            return;
        }

        if ($executed) {
            return;
        }

        if (!$this->bad_word_list) {
            return;
        }

        $message = $this->mod->removecolor($message);
        $multi = 1;
        $bad = false;
        foreach ($this->bad_word_list["normal"] as $value) {
            if (stripos($message, $value[0]) !== false) {
                $bad = true;
                $badword = $value[0];
                $multi = $value[1];
                break;
            }
        }

//        if (!$bad) {
//           foreach ($this->bad_word_list["regexp"] as $value) {
//            if (preg_match("´". $value[0]."´", $message, $subpatterns)) {
//                    $bad = true;
//                    $badword = $subpatterns[0];
//                    $multi = $value[1];
//                    break;
//                }
//            }
//        }

        if ($bad && !$this->permissions->hasPermissions($guid, 'badwords.punishment.ignore')) {
            $this->players[$guid]->badwords_used += $multi;

            $this->logging->info("Badwords: Player '" . $this->players[$guid]->getName() . "' used the bad word '$badword', PID: " . $this->players[$guid]->getPID() . ", GUID: $guid");

            $this->players[$guid]->punishmentScoreAdd($multi);
            $action = $this->registry->punishment->checkScore($this->players[$guid]->getPunishmentScore(), $multi);
            if ($action !== false) {
                if ($action == "warn") {
                    $this->registry->punishment->punish($guid, $action, $multi, array("reason" => $this->mod->getCV("badwords", "kickreason"), "kicker" => false));
                } else {
                    $this->registry->punishment->punish($guid, $action, $multi, array("reason" => $this->mod->getLngString("usedBadWord", array("<WORD>"), array($badword)), "kicker" => false));
                }
            }
        }
    }

    /**
     * Execute the event readConfig.
     *
     * @return void
     */
    public function read() {
        if (!$this->mod->getCV("badwords", "enabled")) {
            return;
        }

        $configdir = $this->mod->getConfigDir();
        $list = $this->mod->getCV("badwords", "words");
        if ($list === false) {
            $this->logging->warning("Badwords: File {$configdir}plugins/badwords.lst could not be opened, badwords will be disabled");
            $this->bad_word_list = false;
            return;
        }
        $normal = array();
        $regexp = array();
        foreach ($list as $value) {
            $value = trim($value);
            if (preg_match('|{([\d.]+)}$|', $value, $subpatterns)) {
                $multi = $subpatterns[1];
                $value = str_replace($subpatterns[0], "", $value);
            } else {
                $multi = 1;
            }
            if (stripos($value, "regexp:") === 0) {
                $regexp[] = array(substr($value, 7), $multi);
            } else {
                $normal[] = array($value, $multi);
            }
        }
        $this->bad_word_list = array("normal" => $normal, "regexp" => $regexp);
    }

    /**
     * On Player Joined
     *
     * @param String $guid
     */
    function playerInit($guid) {
        $this->players[$guid]->badwords_used = 0;
    }
}
