<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Plugins\Banner;

use MAM\Daemon\Core\Utils;

/**
 * Banner plugin
 *
 * sends banner messages all x seconds
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Banner extends Utils {
    /**
     * Count of the current message
     *
     * @var int
     */
    private $banner_current = 0;
    /**
     * Array of all banner messages
     *
     * @var string[]
     */
    private $banner_messages = array();

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->readConfig();

        $this->mod->setDefaultCV('banner', 'enabled', 0);
        $this->mod->setDefaultCV('banner', 'time', 120);
        $this->mod->setDefaultCV('banner', 'logging', 1);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerEvent('parseConfig', 'readconfig', $this);
        $this->mod->registerEvent('mamStarted', 'send', $this);
    }

    /**
     * If !readconfig is executed, the mod will trigger this function
     * This function reads the banner.lst
     */
    public function readConfig() {
        $this->banner_messages = $this->mod->getCV('banner', 'banners');
        $this->banner_messages = array_map('trim', $this->banner_messages);
    }

    /**
     * This Plugin sends the Message and adds new job
     *
     * @return false on fail
     */
    public function send() {
        $this->registry->jobs->addSingleJob($this->mod->getCV('banner', 'time'), array($this, 'send'));

        if (!$this->mod->getCV('banner', 'enabled')) {
            return false;
        }

        if (count($this->players) == 0) {
            return false;
        }

        if ($this->banner_current >= count($this->banner_messages)) {
            $this->banner_current = 0;
        }

        $send = $this->banner_messages[$this->banner_current];

        //Is <NEXTMAP> or <NEXTGT> used
        if (strpos($send, '<NEXTMAP>') !== false || strpos($send, '<NEXTGT>') !== false) {
            list($nextmap, $nextgt) = $this->rcon->rconGetNextMap();
            $nextmap = $this->mod->getLongMapName($nextmap);
            $nextgt = $this->mod->getLongGametype($nextgt);
            $send = str_replace(array('<NEXTMAP>', '<NEXTGT>'), array($nextmap, $nextgt), $send);
        }

        //Is <TIME_DE> or <TIME_EN> or <DATE> used
        if (strpos($send, '<TIME_DE>') !== false || strpos($send, '<TIME_EM>') !== false || strpos($send, '<DATE>') !== false) {
            $send = str_replace(array('<TIME_DE>', '<TIME_EN>', '<DATE>'), array(date('H:i'), date('h:i a'), date('d M Y')), $send);
        }

        //Is <MAP> used
        if (strpos($send, '<MAP>') !== false) {
            $map = implode('', $this->mod->getLastMaps(1));
            $send = str_replace('<MAP>', $this->mod->getLongMapName($map));
        }

        $send = str_replace('<VERSION>', $this->registry->version, $send);

        //send banner
        $this->rcon->rconSay($send);

        if ($this->mod->getCV('banner', 'logging')) {
            $this->logging->info('Banner message was sent: ' . $send);
        }

        $this->banner_current++;
    }
}
