<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+6
 * */

namespace MAM\Plugins\Funmessages;

use MAM\Daemon\Core\Utils;

/**
 * FunMessages
 *
 * This Class includes all funmessages
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class FunMessages extends Utils {

    /**
     * Inits the plugin
     *
     * This function initiates the plugin. This means that it register commands
     * default values, and events. It's important that every plugin has this function
     * Otherwise the plugin exists but can't be used
     */
    public function initPlugin() {
        parent::initPlugin();

        $this->mod->setDefaultCV('funmessages', 'enabled', 0);
    }

    /**
     * Runs the plugin
     *
     * This function is called after all plugins were loaded and dependencies were checked.
     * In this function you can define things which you want to load, when all plugins are fine.
     *
     * If you know that you don't have any dependencies, you can remove this function.
     * Otherwise you should use it.
     *
     * In this function you should call functions like registerCommand, registerEvent, triggerEvent
     */
    public function runPlugin() {
        parent::runPlugin();

        $this->mod->registerCommand('fun', false, 'funmessagesMain', $this);
    }

    /**
     * Shows the funmessages
     *
     * @param   string      $guid       Guid of executing player
     * @param   string[]    $parameters The chatline splitted by " " without !command
     * @return  boolean
     */
    function funmessagesMain($guid, $parameters) {

        if (!$this->mod->getCV('funmessages', 'enabled')) {
            return;
        }

        array_map('strtolower', $parameters);

        if (empty($parameters[0]) || $parameters[0] == 'enabled' || !$this->mod->existsCV('funmessages', $parameters[0])) {
            $messages = implode(', ', array_keys($this->mod->getCVSection('funmessages', array('enabled'))));
            $this->players[$guid]->say($this->mod->getLngString('funmessagesAvailable', array('<LIST>'), array($messages)));
            return;
        }

        $search = array(
            '<PLAYER_NAME>',
            '<PLAYER_TEAM>',
            '<PLAYER_LASTKILLER>',
            '<PLAYER_LASTVICTIM>',
            '<PLAYER_LASTWEAPONKILL>',
            '<PLAYER_LASTWEAPONDEATH>',
            '<PLAYER_KILLS>',
            '<PLAYER_DEATHS>',
            '<PLAYER_TEAMKILLS>',
            '<PLAYER_HEADSHOTS>',
            '<TIME>',
            '<PLAYER_COUNTRY_CODE>',
            '<PLAYER_COUNTRY_NAME>',
        );

        $player = &$this->players[$guid];

        if ($player->getLastVictim() == $guid) {
            $lastvictim = $this->mod->getLngString('himself');
        } else {
            $lastvictim = ($player->getLastVictim() != false) ? $this->players[$player->getLastVictim()]->getName() : $this->mod->getLngString('nobody');
        }
        if ($player->getLastKiller() == $guid) {
            $lastkiller = $this->mod->getLngString('himself');
        } else {
            $lastkiller = ($player->getLastKiller() != false) ? $this->players[$player->getLastKiller()]->getName() : $this->mod->getLngString('nobody');
        }

        try {
            $geoip_data = $this->geoip->getLocationData($this->players[$guid]->getIP());

            if (isset($geoip_data['country']['iso_code'])) {
                $country_code = $geoip_data['country']['iso_code'];
            } else {
                $country_code = 'xy';
            }

            if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'english') {
                $country_name = $geoip_data['country']['names']['en'];
            } else if (isset($geoip_data['country']['names']) && $this->mod->getCV('daemon', 'language') == 'german') {
                $country_name = $geoip_data['country']['names']['de'];
            } else {
                $country_name = 'cosmic space (using IPv9)';
            }
        } catch (\MAM\Daemon\Libraries\GeoIP\GeoIpException $e) {
            $this->logging->notice('Geoip: ' . $e);
            $country_name = 'cosmic space (using IPv9)';
            $country_code = 'xy';
        }

        $replace = array(
            $player->getName(),
            $this->mod->getLongTeamName($player->getTeam()),
            $lastkiller,
            $lastvictim,
            $this->mod->getWeaponInfo($player->lastweaponkill)['name'],
            $this->mod->getWeaponInfo($player->lastweapondeath)['name'],
            $player->kills,
            $player->deaths,
            $player->teamkills,
            $player->headshots,
            date('H:i'),
            $country_code,
            $country_name,
        );

        $this->rcon->rconSay(str_replace($search, $replace, $this->mod->getCV('funmessages', $parameters[0])));
    }
}
