<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Plugins\Tcp;

/**
 * This class creates the TCP Connection.
 *
 * @author manu
 */
class TcpConnection {
    /**
     * con
     *
     * @var boolean
     */
    private $con = false;
    /**
     * created
     *
     * @var int
     */
    private $created = 0;
    /**
     * login Started
     *
     * @var int
     */
    private $loginStarted = 0;
    /**
     * last command
     *
     * @var int
     */
    private $lastcommand = false;
    /**
     * status
     *
     * @var string
     */
    private $status = "loggedout";
    /**
     * user
     *
     * @var int
     */
    private $user = false;
    /**
     * IP-Address
     *
     * @var string
     */
    private $ip;
    /**
     * Port of the IP-Address
     *
     * @var int
     */
    private $port;
    /**
     * Buffer
     *
     * @var string
     */
    private $buffer = "";
    /**
     * Group
     *
     * @var mixed
     */
    private $group = false;
    /**
     * Closed
     *
     * @var mixed
     */
    public $closed = false;
    /**
     * Dummy Player
     *
     * @var mixed
     */
    public $dummyplayer;
    /**
     * Sent
     *
     * @var int
     */
    private $sent = 0;
    /**
     * is received
     *
     * @var int
     */
    private $received = 0;
    /**
     * is result
     *
     * @var mixed
     */
    private $isResult = false;
    /**
     * Class for mam related stuff
     *
     * @var \MAM\Daemon\Core\Mod
     */
    protected $mod;
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     */
    protected $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     */
    protected $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;

    /**
     * Registry
     * @var \MAM\Daemon\Core\Registry
     */
    protected $registry;

    /**
     * Constructor
     *
     * @param mixed $socket
     */
    public function __construct($socket) {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = &$registry->players;
        $this->rcon = $registry->rcon;
        $this->logging = $registry->logging;
        $this->registry = $registry;

        $this->con = $socket;
        $this->created = time();
        $this->loginStarted = time();
        socket_getpeername($this->con, $this->ip, $this->port);

        $this->logging->info("TCP-Query: New connection from $this->ip on port $this->port");

        $this->welcome();
        $this->getUserName();
    }

    /**
     * Destruct
     */
    public function __destruct() {
        if (is_resource($this->con)) {
            $this->close();
        }
    }

    /**
     * Welcome Message for TCP User.
     */
    private function welcome() {
        $timeout = $this->mod->getCV("tcp_query", "logintimeout");
        $msg = "Welcome to Manu-Admin-Mod TCP-Interface" . PHP_EOL;
        $msg .= "Your IP is $this->ip and will be logged for security reasons" . PHP_EOL;
        $msg .= "You have $timeout seconds to authentificate..." . PHP_EOL;
        $this->write($msg);
    }

    /**
     * Reads in the tcp buffer
     *
     * @return void
     */
    private function read() {
        $read = @socket_read($this->con, 9999, PHP_BINARY_READ);


        $this->received += strlen($read);
        //WIN  , LINUX, FreeBSD
        if ($read === '' || !in_array(@socket_last_error($this->con), array(10035, 11, 35))) {
            $this->close();
            return;
        }

        $this->buffer .= $read;

        if (strpos($this->buffer, "\n") !== false) {
            $parts = explode("\n", $this->buffer, 2);
            $cmd = trim($parts[0]);
            if (!empty($cmd)) {
                $this->input($cmd);
            }
            $this->buffer = "";
        }
    }

    /**
     * Writes into socket
     *
     * @param string $msg The Message
     * @param string $type which window in tcp console
     */
    public function write($msg, $type = \MAM\Plugins\Tcp\TcpLevel::MT_CONSOLE) {


        $isCmd = ($this->isResult) ? chr(1) : chr(0);
        $msg = chr($type) . $isCmd . $msg;
        if ($this->mod->getCV("tcp_query", "asciiColors")) {
            $msg = \MAM\Daemon\Libraries\Logging\LogHandler::colorOutput($msg);
        }

        @socket_write($this->con, $msg, strlen($msg));
        $this->sent += strlen($msg);
    }

    /**
     * Executes the everyTime event.
     */
    public function everyTime() {
        $this->read();

        $timeout = $this->mod->getCV("tcp_query", "logintimeout");
        if ($this->status == "loggedout" && time() - $this->loginStarted > $timeout) {
            $this->write("Your $timeout seconds are over, please try again" . PHP_EOL);
            $this->close();
        }
    }

    /**
     * Close the connection.
     *
     * @return void
     */
    private function close() {
        if ($this->closed) {
            return;
        }
        $this->write("Bye, see you later :D" . PHP_EOL);
        $this->closed = true;
        if ($this->status == "loggedout") {
            $this->user = "NOBODY";
        } else {
            $this->status = "loggedout";
            $this->rcon->rconSay("TCP-Admin ($this->user) has disconnected");
        }
        $this->logging->info("TCP-Query: Connection closed: $this->user ($this->ip:$this->port)");
        socket_close($this->con);
    }

    /**
     * Input from the TCP Console
     *
     * @param String $input
     */
    private function input($input) {
        if ($this->status == "loggedout") {
            if ($this->user == false) {
                $this->user = $input;
                $this->getPassword();
            } else {
                if ($this->login($this->user, $input)) {
                    $this->write("You have been successfully logged in as $this->user " . PHP_EOL . "Have fun :D" . PHP_EOL);
                } else {
                    $this->write("Login failed, please try again..." . PHP_EOL);
                    $this->user = false;
                    $this->getUserName();
                }
            }
        } else {
            $this->isResult = true;
            //quit command
            if (strtolower($input) == "quit" || strtolower($input) == "exit") {
                $this->close();
            } elseif (strtolower($input) == "logout") {
                $this->logout();
            } elseif (strtolower($input) == "info") {
                $this->connectionInfo();
            } elseif (strtolower($input) == "lc" && in_array($this->group, $this->mod->getCV("tcp_query", "admingroups"))) {
                $connections = $this->registry->tcpquery->getConnections();
                $msg = "";
                foreach ($connections as $connection) {
                    $msg .= str_pad($connection["id"], 3, " ", STR_PAD_LEFT) . ": ";
                    $msg .= str_pad($connection["ip"] . ":" . $connection["port"], 21, " ", STR_PAD_RIGHT) . " ";
                    $msg .= $connection["user"] . PHP_EOL;
                }
                $this->write($msg);
            } elseif (substr(strtolower($input), 0, 5) == "kick " && in_array($this->group, $this->mod->getCV("tcp_query", "admingroups"))) {
                $id = explode(" ", $input);
                $this->logging->info("User '$this->user' used command kick");
                if ($this->registry->tcpquery->kick($id[1])) {
                    $this->write("User has been kicked" . PHP_EOL);
                } else {
                    $this->write("Error..." . PHP_EOL);
                }
            } elseif (preg_match('|^getinfo([^\w].*)?$|', $input)) {
                if (strlen($input) <= 8) {
                    $this->write("Possible: players, maps");
                } else {
                    switch (substr($input, 8)) {
                        case "players":
                            $this->writePlayers();
                            break;
                        case "maps":
                            $this->writeMaps();
                            break;
                        default:
                            $this->write("error");
                    }
                }
            } elseif (substr(strtolower($input), 0, 5) == "rcon " && $this->dummyplayer->isAllowedToExec("rcon")) {
                $rcon = substr($input, 5);
                $return = $this->rcon->rconRcon($rcon);
                $this->write($return);
            } else {
                if ($input[0] == $this->mod->getCV("daemon", "cmdprefix")) {
                    $this->players["TCPUSER"] = & $this->dummyplayer;
                    $this->mod->executeCommand($input, "TCPUSER", $executed);
                    if (!$executed && !$this->mod->getCV("daemon", "wrongcmdmessage")) {
                        $this->write("Unrecognized command: " . $input . PHP_EOL);
                    }
                    unset($this->players["TCPUSER"]);
                } else {
                    if ($this->mod->getCV("tcp_query", "defaulaction") == "say") {
                        $msg = str_replace("<ADMIN>", $this->user, $this->mod->getCV("tcp_query", "sayprefix")) . $input;
                        $this->logging->info("TCP-Query: User '$this->user' says: '$input' ($this->ip:$this->port)");
                        $this->rcon->rconSay($msg);
                    } elseif ($this->mod->getCV("tcp_query", "defaulaction") == "rcon") {
                        $return = $this->rcon->rconRcon($input);
                        $this->logging->info("TCP-Query: User '$this->user' executed RCON command: '$input' ($this->ip:$this->port)");
                        $this->write($return);
                    }
                }
            }
            $this->isResult = false;
        }
    }

    /**
     * get Username
     */
    private function getUserName() {
        $this->write("User: " . PHP_EOL);
    }

    /**
     * get Password
     */
    private function getPassword() {
        $this->write("Password: " . PHP_EOL);
    }

    /**
     * The Login into the game.
     *
     * @param String $user
     * @param String $password
     * @return boolean
     */
    private function login($user, $password) {
        if ($this->registry->tcpquery->isValidLogin($user, $password, $group)) {
            $this->group = $group;  //Is changed by function isValidLogin
            $this->user = $user;    //Is changed by function isValidLogin
            $this->status = "loggedin";
            $this->logging->info("TCP-Query: User logged in: $this->user ($this->ip:$this->port)");
            $this->rcon->rconSay("TCP-Admin ($this->user) has connected");
            $this->dummyplayer = new \MAM\Plugins\Tcp\Player($this->user, $this->group, $this);
            return true;
        }
        return false;
    }

    /**
     * The logout message before the user quit the connection
     */
    private function logout() {
        $this->loginStarted = time();
        $this->status = "loggedout";
        $this->write("You have been successfully logged out!" . PHP_EOL . PHP_EOL);
        $this->logging->info("TCP-Query: User logged out: $this->user ($this->ip:$this->port)");
        $this->rcon->rconSay("TCP-Admin ($this->user) has disconnected");
        $this->user = false;
        $this->group = "";
        $this->welcome();
        $this->getUserName();
    }

    /**
     * Is the user logged in
     *
     * @return boolean
     */
    public function loggedin() {
        if ($this->status == "loggedin") {
            return true;
        }
        return false;
    }

    /**
     * gets the ip address
     *
     * @return string
     */
    public function getIP() {
        return $this->ip;
    }

    /**
     * gets the port
     *
     * @return int
     */
    public function getPort() {
        return $this->port;
    }

    /**
     * get User
     *
     * @return string
     */
    public function getUser() {
        if ($this->loggedin()) {
            return $this->user;
        }
        return "NOT LOGGED IN";
    }

    /**
     * The kick message for a TCP user.
     */
    public function kick() {
        $this->write("You have been kicked by Admin" . PHP_EOL);
        $this->logging->info("TCP-Query: Connection of '" . $this->getUser() . "' ($this->ip:$this->port) got kicked by admin");
        $this->close();
    }

    /**
     * The Info of the Connection.
     */
    private function connectionInfo() {
        $ip = $this->getIP();
        $port = $this->getPort();
        $user = $this->getUser();
        $group = $this->mod->getLongGroupName($this->group);
        $time = \MAM\Daemon\Libraries\Helper\Helper::makeUptime2(time() - $this->created);

        $text = "Connection info:" . PHP_EOL . "================" . PHP_EOL;
        $text .= "Username:        $user ($group)" . PHP_EOL;
        $text .= "Address:         $ip:$port" . PHP_EOL;
        $text .= "Connection time: $time" . PHP_EOL;
        $text .= "Bytes read:      " . \MAM\Daemon\Libraries\Helper\Helper::size($this->received) . PHP_EOL;
        $text .= "Bytes sent:      " . \MAM\Daemon\Libraries\Helper\Helper::size($this->sent) . PHP_EOL;


        $this->write($text);
    }

    /**
     * writes the Maps.
     */
    private function writeMaps() {
        $str = "Game: " . $this->mod->getGame() . PHP_EOL;
        $lastmaps = $this->mod->getLastMaps(1);
        $str .= "Current: " . $this->mod->getLongMapName($lastmaps[0]);
        $str .= " (" . $this->mod->getLongGametype($this->mod->getCurrentGametype()) . ")" . PHP_EOL;
        $maps = $this->mod->getMaps();
        sort($maps);
        foreach ($maps as $short) {
            $str .= "$short\t";
            $str .= $this->mod->getLongMapName($short) . PHP_EOL;
        }
        $str = trim($str);

        $this->write($str);
    }

    /**
     * writes the Players
     */
    private function writePlayers() {
        $str = "";
        foreach (array_keys($this->players) as $guid) {
            $str .= $this->players[$guid]->getPID();
            $str .= "\t" . $this->players[$guid]->getName();
            $str .= "\t";
            $str .= $this->mod->getLongGroupName($this->players[$guid]->getGroup());
            $str .= "\t$guid\t0" . PHP_EOL;
        }
        $this->write(trim($str));
    }
}
